@@ -15,3 +15,11 @@ spaMM.options(rankMethod=".rankinfo")
 fv3 <- fitme(y~int+ b1+b2+c1+c2+c3,data=donn)$fv ## fixef is not unique, but fitted values must be equivalent
 spaMM.options(rankMethod="qr")
 testthat::expect_true( max(apply(cbind(fv1,fv2,fv3),1L,var))<1e-16)
+
+donn$dummy<- c(0,0,0,1,1,1)
+chk <- try(nlme::lme(y~int+ b1+b2+c1+c2+c3, data = donn, random = ~ 1 | dummy), silent=TRUE)
+testthat::test_that("check claim, from the Description of spaMM::rankinfo, that nlme::lme does not handle singular X",
+                    testthat::expect_true(inherits(chk,"try-error"))
+)
+# chk <- try(lme4::lmer(y~int+ b1+b2+c1+c2+c3 + (1 | dummy), data = donn), silent=TRUE), warnings, but this fits contrary to lme.
+
